@@ -41,6 +41,138 @@
 	also HDR.NRec is defined here
 */
 
+#if defined(WITH_ATF)
+EXTERN_C void sopen_atf_read(HDRTYPE* hdr) {
+
+		ifseek(hdr,0,SEEK_SET);
+		size_t ll;
+		char *line = NULL;
+		ssize_t nc;
+		typeof (hdr->NS) k;
+
+		// first line - skip: contains "ATF\t1.0" or alike
+		nc = getline(&line, &ll, hdr->FILE.FID);
+
+		if (VERBOSE_LEVEL>7) fprintf(stdout,"ATF line <%s>\n",line);
+
+		// 2nd line: number of rows and colums
+		if (line!=NULL) { free(line); line=NULL; }  // allocate line buffer as needed
+		nc = getline(&line, &ll, hdr->FILE.FID);
+		char *str = line;
+		hdr->NRec = strtoul(str,&str,10);
+		hdr->SPR  = 1;
+		hdr->NS   = strtoul(str,&str,10);
+		hdr->SampleRate = 1.0;
+		hdr->CHANNEL = (CHANNEL_TYPE*) realloc(hdr->CHANNEL, hdr->NS * sizeof(CHANNEL_TYPE));
+
+		if (VERBOSE_LEVEL>7) fprintf(stdout,"ATF line <%s>\n",line);
+
+		// 3rd line: channel label
+		if (line!=NULL) { free(line); line=NULL; }  // allocate line buffer as needed
+		nc  = getline(&line, &ll, hdr->FILE.FID);
+
+		if (VERBOSE_LEVEL>7) fprintf(stdout,"ATF line <%s>\n",line);
+
+		typeof(hdr->NS) TIMECHANNEL = 0;
+
+		/* TODO:
+			distinguish between ATF files with single channel and multiple sections and
+			and ATF files with single section and multiple channels
+		 */
+		int flag_sections_only=1;
+
+		str = strtok(line,"\t\n\r");
+		for (k = 0; k < hdr->NS; k++) {
+			CHANNEL_TYPE *hc = hdr->CHANNEL+k;
+			//init_channel(hc);
+			hc->GDFTYP 	= 17; // float64
+			hc->PhysMin	= -1e9;
+			hc->PhysMax	= +1e9;
+			hc->DigMin	= -1e9;
+			hc->DigMax	= +1e9;
+			hc->Cal		= 1.0;
+			hc->Off		= 0.0;
+
+			//hc->Label[0]  = '\0';
+			hc->OnOff	= 1;
+			hc->LeadIdCode	= 0;
+			hc->Transducer[0] = '\0';
+			hc->PhysDimCode	= 0;
+#ifdef MAX_LENGTH_PHYSDIM
+			hc->PhysDim[0]	= '?';
+#endif
+			hc->TOffset	= 0.0;
+			hc->HighPass	= NAN;
+			hc->LowPass	= NAN;
+			hc->Notch	= NAN;
+			hc->XYZ[0]	= 0;
+			hc->XYZ[1]	= 0;
+			hc->XYZ[2]	= 0;
+			hc->Impedance	= NAN;
+
+			hc->bufptr 	= NULL;
+			hc->SPR 	= 1;
+			hc->bi8	 	= k * GDFTYP_BITS[hc->GDFTYP];
+			hc->bi	 	= hc->bi8 / 8;
+			hdr->AS.bpb    += GDFTYP_BITS[hc->GDFTYP]/8;
+
+			if (str != NULL) {
+				size_t len = strlen(str);
+				strncpy(hc->Label, str+1, len-2); // do not copy quotes
+				if (strncmp(hc->Label,"Section",7)) flag_sections_only=0;
+
+				// extract physical units enclosed in parenthesis "Label (units)"
+				str = strchr(str+1,'(');
+				if (str != NULL) {
+					char *str2 = strchr(str,')');
+					if (str2 != NULL) {
+						*str2 = 0;
+						hc->PhysDimCode = PhysDimCode(str+1);
+					}
+				}
+				if (!strncasecmp(hc->Label,"time",4)) {
+					TIMECHANNEL = k+1;
+					hc->OnOff   = 0;   // mark channel as containing the time axis
+					if (k==0) hdr->SampleRate /= PhysDimScale(hc->PhysDimCode);
+				}
+			}
+			// extract next label
+			str = strtok(NULL,"\t\n\r");
+
+			if (VERBOSE_LEVEL>7) fprintf(stdout,"ATF label #%i:<%s>\n",(int)k,hc->Label);
+
+		}
+		hdr->HeadLen = iftell(hdr);
+
+		// 4th line: 1st sample of time channel
+		if (line!=NULL) { free(line); line=NULL; }  // allocate line buffer as needed
+		nc  = getline(&line, &ll, hdr->FILE.FID);
+		double t1 = strtod(line, &str);
+
+		// 5th line: 2nd sample of time channel
+		if (line!=NULL) { free(line); line=NULL; }  // allocate line buffer as needed
+		nc  = getline(&line, &ll, hdr->FILE.FID);
+		double t2 = strtod(line, &str);
+
+		hdr->SampleRate /= (t2-t1);
+		if (line!=NULL) { free(line); line=NULL; }  // allocate line buffer as needed
+
+		/*
+		if ((size_t)(hdr->NRec * hdr->SPR) <= (ln+1) ) {
+			hdr->NRec = max(1024, ln*2);
+			hdr->AS.rawdata = realloc(hdr->AS.rawdata, hdr->NRec * hdr->SPR * hdr->AS.bpb);
+		}
+
+			TODO:
+			 this marks that no data has been read, and
+			 hdr->SampleRate, are not defined
+		 */
+		//biosigERROR(hdr, B4C_DATATYPE_UNSUPPORTED, "support for ATF files not complete");
+
+		hdr->AS.rawdata = NULL;
+
+}
+
 EXTERN_C void sread_atf(HDRTYPE* hdr) {
 
 	if (VERBOSE_LEVEL>6) fprintf(stdout,"SREAD ATF [%i,%i]\n",(unsigned)hdr->NRec, (unsigned)hdr->SPR);
@@ -89,6 +221,7 @@ EXTERN_C void sread_atf(HDRTYPE* hdr) {
 	hdr->AS.first  = 0;
 	hdr->AS.length = hdr->NRec;
 }
+#endif
 
 EXTERN_C void sopen_abf_read(HDRTYPE* hdr) {	
 /*
@@ -101,11 +234,11 @@ EXTERN_C void sopen_abf_read(HDRTYPE* hdr) {
 
 	if (VERBOSE_LEVEL>7) fprintf(stdout,"sopen_abf_read 101\n");
 
-	if (VERBOSE_LEVEL>7) fprintf(stderr,"%s (line %i) %li %i %i %li\n",__FILE__,__LINE__,
+	if (VERBOSE_LEVEL>7) fprintf(stderr,"%s (line %i) %"PRIiPTR" %i %i %"PRIiPTR"\n",__FILE__,__LINE__,
                 sizeof(struct ABFFileHeader),ABF_OLDHEADERSIZE, ABF_HEADERSIZE,
                 offsetof(struct ABFFileHeader, lHeaderSize));
 
-	if (VERBOSE_LEVEL>7) fprintf(stderr,"%s (line %i) %i %li %i\n",__FILE__,__LINE__,
+	if (VERBOSE_LEVEL>7) fprintf(stderr,"%s (line %i) %i %"PRIiPTR" %i\n",__FILE__,__LINE__,
 		ABF_BLOCKSIZE, offsetof(struct ABFFileHeader, lDataSectionPtr),
 		ABF_BLOCKSIZE*lei32p(hdr->AS.Header + offsetof(struct ABFFileHeader, lDataSectionPtr)) );
 
@@ -532,35 +665,35 @@ EXTERN_C void sopen_abf2_read(HDRTYPE* hdr) {
 
 			fprintf(stdout,"DataSection.uBlockIndex: \t%i\n",leu32p(hdr->AS.Header + offsetof(struct ABF_FileInfo, DataSection.uBlockIndex)));
 			fprintf(stdout,"DataSection.uBytes:      \t%i\n",leu32p(hdr->AS.Header + offsetof(struct ABF_FileInfo, DataSection.uBytes)));
-			fprintf(stdout,"DataSection.llNumEntries:\t%li\n",lei64p(hdr->AS.Header + offsetof(struct ABF_FileInfo, DataSection.llNumEntries)));
+			fprintf(stdout,"DataSection.llNumEntries:\t%"PRIi64"\n",lei64p(hdr->AS.Header + offsetof(struct ABF_FileInfo, DataSection.llNumEntries)));
 
 			fprintf(stdout,"TagSection.uBlockIndex:  \t%i\n",leu32p(hdr->AS.Header + offsetof(struct ABF_FileInfo, TagSection.uBlockIndex)));
 			fprintf(stdout,"TagSection.uBytes:       \t%i\n",leu32p(hdr->AS.Header + offsetof(struct ABF_FileInfo, TagSection.uBytes)));
-			fprintf(stdout,"TagSection.llNumEntries: \t%li\n",lei64p(hdr->AS.Header + offsetof(struct ABF_FileInfo, TagSection.llNumEntries)));
+			fprintf(stdout,"TagSection.llNumEntries: \t%"PRIi64"\n",lei64p(hdr->AS.Header + offsetof(struct ABF_FileInfo, TagSection.llNumEntries)));
 
 			fprintf(stdout,"ScopeSection.uBlockIndex: \t%i\n",leu32p(hdr->AS.Header + offsetof(struct ABF_FileInfo, ScopeSection.uBlockIndex)));
 			fprintf(stdout,"ScopeSection.uBytes:      \t%i\n",leu32p(hdr->AS.Header + offsetof(struct ABF_FileInfo, ScopeSection.uBytes)));
-			fprintf(stdout,"ScopeSection.llNumEntries:\t%li\n",lei64p(hdr->AS.Header + offsetof(struct ABF_FileInfo, ScopeSection.llNumEntries)));
+			fprintf(stdout,"ScopeSection.llNumEntries:\t%"PRIi64"\n",lei64p(hdr->AS.Header + offsetof(struct ABF_FileInfo, ScopeSection.llNumEntries)));
 
 			fprintf(stdout,"DeltaSection.uBlockIndex: \t%i\n",leu32p(hdr->AS.Header + offsetof(struct ABF_FileInfo, DeltaSection.uBlockIndex)));
 			fprintf(stdout,"DeltaSection.uBytes:      \t%i\n",leu32p(hdr->AS.Header + offsetof(struct ABF_FileInfo, DeltaSection.uBytes)));
-			fprintf(stdout,"DeltaSection.llNumEntries:\t%li\n",lei64p(hdr->AS.Header + offsetof(struct ABF_FileInfo, DeltaSection.llNumEntries)));
+			fprintf(stdout,"DeltaSection.llNumEntries:\t%"PRIi64"\n",lei64p(hdr->AS.Header + offsetof(struct ABF_FileInfo, DeltaSection.llNumEntries)));
 
 			fprintf(stdout,"VoiceTagSection.uBlockIndex: \t%i\n",leu32p(hdr->AS.Header + offsetof(struct ABF_FileInfo, VoiceTagSection.uBlockIndex)));
 			fprintf(stdout,"VoiceTagSection.uBytes:      \t%i\n",leu32p(hdr->AS.Header + offsetof(struct ABF_FileInfo, VoiceTagSection.uBytes)));
-			fprintf(stdout,"VoiceTagSection.llNumEntries:\t%li\n",lei64p(hdr->AS.Header + offsetof(struct ABF_FileInfo, VoiceTagSection.llNumEntries)));
+			fprintf(stdout,"VoiceTagSection.llNumEntries:\t%"PRIi64"\n",lei64p(hdr->AS.Header + offsetof(struct ABF_FileInfo, VoiceTagSection.llNumEntries)));
 
 			fprintf(stdout,"SynchArraySection.uBlockIndex: \t%i\n",leu32p(hdr->AS.Header + offsetof(struct ABF_FileInfo, SynchArraySection.uBlockIndex)));
 			fprintf(stdout,"SynchArraySection.uBytes:      \t%i\n",leu32p(hdr->AS.Header + offsetof(struct ABF_FileInfo, SynchArraySection.uBytes)));
-			fprintf(stdout,"SynchArraySection.llNumEntries:\t%li\n",lei64p(hdr->AS.Header + offsetof(struct ABF_FileInfo, SynchArraySection.llNumEntries)));
+			fprintf(stdout,"SynchArraySection.llNumEntries:\t%"PRIi64"\n",lei64p(hdr->AS.Header + offsetof(struct ABF_FileInfo, SynchArraySection.llNumEntries)));
 
 			fprintf(stdout,"AnnotationSection.uBlockIndex: \t%i\n",leu32p(hdr->AS.Header + offsetof(struct ABF_FileInfo, AnnotationSection.uBlockIndex)));
 			fprintf(stdout,"AnnotationSection.uBytes:      \t%i\n",leu32p(hdr->AS.Header + offsetof(struct ABF_FileInfo, AnnotationSection.uBytes)));
-			fprintf(stdout,"AnnotationSection.llNumEntries:\t%li\n",lei64p(hdr->AS.Header + offsetof(struct ABF_FileInfo, AnnotationSection.llNumEntries)));
+			fprintf(stdout,"AnnotationSection.llNumEntries:\t%"PRIi64"\n",lei64p(hdr->AS.Header + offsetof(struct ABF_FileInfo, AnnotationSection.llNumEntries)));
 
 			fprintf(stdout,"StatsSection.uBlockIndex: \t%i\n",leu32p(hdr->AS.Header + offsetof(struct ABF_FileInfo, StatsSection.uBlockIndex)));
 			fprintf(stdout,"StatsSection.uBytes:      \t%i\n",leu32p(hdr->AS.Header + offsetof(struct ABF_FileInfo, StatsSection.uBytes)));
-			fprintf(stdout,"StatsSection.llNumEntries:\t%li\n",lei64p(hdr->AS.Header + offsetof(struct ABF_FileInfo, StatsSection.llNumEntries)));
+			fprintf(stdout,"StatsSection.llNumEntries:\t%"PRIi64"\n",lei64p(hdr->AS.Header + offsetof(struct ABF_FileInfo, StatsSection.llNumEntries)));
 
 		}
 
